package com.huigou.bpmc.chart.controller;

import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;

import com.huigou.bpmc.chart.application.BPMCFlowChartApplication;
import com.huigou.bpmc.chart.domain.FlowInterfaceKind;
import com.huigou.bpmc.chart.domain.FlowRuleKind;
import com.huigou.bpmc.chart.domain.model.ProcessAreas;
import com.huigou.bpmc.chart.domain.model.ProcessNode;
import com.huigou.bpmc.chart.domain.model.ProcessNodeFunction;
import com.huigou.bpmc.chart.domain.model.ProcessNodeLine;
import com.huigou.bpmc.chart.domain.model.ProcessNodeTemp;
import com.huigou.bpmc.chart.domain.query.ProcessNodeFunctionQueryRequest;
import com.huigou.bpmc.model.ObjectKindEnum;
import com.huigou.bpmc.process.domain.model.BusinessProcess;
import com.huigou.uasp.annotation.ControllerMapping;
import com.huigou.uasp.client.CommonController;
import com.huigou.util.SDO;
import com.huigou.util.StringUtil;

/**
 * 流程图绘制
 *
 * @author xx
 * @version V1.0
 * @ClassName: FlowNodeController
 * @date 2017-03-06 14:49
 */
@Controller
@ControllerMapping("opFlowChart")
public class BPMCFlowChartController extends CommonController {

    @Autowired
    private BPMCFlowChartApplication bpmcFlowChartApplication;

    protected String getPagePath() {
        return "/biz/bpmc/flowchart/";
    }

    public String forwardFlowchart() {
        SDO sdo = this.getSDO();
        String businessProcessId = sdo.getString("businessProcessId");
        BusinessProcess process = bpmcFlowChartApplication.loadBusinessProcess(businessProcessId);
        this.putAttribute("flowNodeKind", ObjectKindEnum.getFlowNodeKind());
        this.putAttribute("flowRuleKind", FlowRuleKind.getData());
        this.putAttribute("flowInterfaceKind", FlowInterfaceKind.getData());
        return this.forward("flowchart", process);
    }

    public String queryFlowNodesAndLines() {
        SDO sdo = this.getSDO();
        String businessProcessId = sdo.getString("businessProcessId");
        Map<String, Object> data = bpmcFlowChartApplication.queryFlowNodesAndLines(businessProcessId);
        return this.toResult(data);
    }

    public String showLoadFlowNode() {
        SDO sdo = this.getSDO();
        String viewId = sdo.getString("viewId");
        String businessProcessId = sdo.getString("businessProcessId");
        String objectKindCode = sdo.getString("objectKindCode");
        ProcessNode node = bpmcFlowChartApplication.loadFlowNodeByViewId(businessProcessId, viewId);
        if (node != null) {
            objectKindCode = node.getObjectKindCode();
        }
        this.putAttribute("flowNodeKind", ObjectKindEnum.getFlowNodeKind());
        this.putAttribute("flowRuleKind", FlowRuleKind.getData());
        this.putAttribute("flowInterfaceKind", FlowInterfaceKind.getData());
        this.putAttribute("viewId", viewId);
        this.putAttribute("objectKindCode", objectKindCode);
        return forward(String.format("node%sDetail", objectKindCode), node);
    }

    public String saveFlowNodesAndLines() {
        SDO sdo = this.getSDO();
        String businessProcessId = sdo.getString("businessProcessId");
        String chartDirection = sdo.getString("chartDirection");
        List<ProcessNode> nodes = sdo.getList("nodes", ProcessNode.class);
        List<ProcessNodeLine> lines = sdo.getList("lines", ProcessNodeLine.class);
        List<ProcessAreas> areas = sdo.getList("areas", ProcessAreas.class);
        bpmcFlowChartApplication.saveFlowNodesAndLines(businessProcessId, chartDirection, nodes, lines, areas);
        return success();
    }

    public String saveFlowNode() {
        SDO sdo = this.getSDO();
        String businessProcessId = sdo.getString("businessProcessId");
        ProcessNodeTemp node = sdo.toObject(ProcessNodeTemp.class);
        node = bpmcFlowChartApplication.saveFlowNode(businessProcessId, node);
        return success(node);
    }

    public String deleteFlowChart() {
        SDO sdo = this.getSDO();
        String businessProcessId = sdo.getString("businessProcessId");
        bpmcFlowChartApplication.deleteFlowChart(businessProcessId);
        return success();
    }

    public String queryProcessNodeFunction() {
        SDO sdo = this.getSDO();
        ProcessNodeFunctionQueryRequest queryRequest = sdo.toQueryRequest(ProcessNodeFunctionQueryRequest.class);
        Map<String, Object> data = bpmcFlowChartApplication.queryProcessNodeFunction(queryRequest);
        return toResult(data);
    }

    public String showInsertProcessNodeFunction() {
        SDO sdo = this.getSDO();
        return forward("processNodeFunctionDetail", sdo);
    }

    public String saveProcessNodeFunction() {
        SDO sdo = this.getSDO();
        ProcessNodeFunction processNodeFunction = sdo.toObject(ProcessNodeFunction.class);
        String id = bpmcFlowChartApplication.saveProcessNodeFunction(processNodeFunction);
        return success(id);
    }

    public String showLoadProcessNodeFunction() {
        SDO sdo = this.getSDO();
        String id = sdo.getId();
        ProcessNodeFunction processNodeFunction = bpmcFlowChartApplication.loadProcessNodeFunction(id);
        return forward("processNodeFunctionDetail", processNodeFunction);
    }

    public String deleteProcessNodeFunction() {
        SDO sdo = this.getSDO();
        List<String> ids = sdo.getIds();
        bpmcFlowChartApplication.deleteProcessNodeFunction(ids);
        return success();
    }

    public String updateProcessNodeFunctionSequence() {
        SDO sdo = this.getSDO();
        Map<String, Integer> map = sdo.getStringMap("data");
        bpmcFlowChartApplication.updateProcessNodeFunctionSequence(map);
        return success();
    }

    public String showViewFlowchart() {
        SDO sdo = this.getSDO();
        String businessProcessId = sdo.getString("businessProcessId");
        BusinessProcess process = null;
        if (StringUtil.isNotBlank(businessProcessId)) {
            process = bpmcFlowChartApplication.loadBusinessProcess(businessProcessId);
        } else {
            String code = sdo.getString("code");
            process = bpmcFlowChartApplication.loadBusinessProcessByCode(code);
        }
        this.putAttribute("businessProcessId", process.getId());
        this.putAttribute("processRemark", process.getFlowAim());
        return this.forward("flowchartUsed", process);
    }

    public String queryFlowNodesAndLinesAndFunction() {
        SDO sdo = this.getSDO();
        String businessProcessId = sdo.getString("businessProcessId");
        Map<String, Object> data = bpmcFlowChartApplication.queryFlowNodesAndLinesAndFunction(businessProcessId);
        return this.toResult(data);
    }
}
