package com.huigou.bpmc.function.application.impl;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import com.huigou.bpmc.function.application.BizFunctionApplication;
import com.huigou.bpmc.function.application.FunctionGroupPermissionsApplication;
import com.huigou.bpmc.function.domain.model.Functions;
import com.huigou.bpmc.function.domain.model.FunctionsDetails;
import com.huigou.bpmc.function.domain.model.FunctionsDetailsDto;
import com.huigou.bpmc.function.domain.model.FunctionsGroup;
import com.huigou.bpmc.function.domain.model.FunctionsGroupDto;
import com.huigou.bpmc.function.domain.query.FunctionsDetailsQueryRequest;
import com.huigou.bpmc.function.domain.query.FunctionsGroupQueryRequest;
import com.huigou.bpmc.function.domain.query.FunctionsQueryRequest;
import com.huigou.bpmc.function.repository.FunctionsDetailsRepository;
import com.huigou.bpmc.function.repository.FunctionsGroupRepository;
import com.huigou.bpmc.function.repository.FunctionsRepository;
import com.huigou.context.MessageSourceContext;
import com.huigou.context.Operator;
import com.huigou.context.ThreadLocalUtil;
import com.huigou.data.domain.model.CommonDomainConstants;
import com.huigou.data.query.model.QueryDescriptor;
import com.huigou.data.query.model.QueryModel;
import com.huigou.uasp.bmp.common.application.BaseApplication;
import com.huigou.uasp.bmp.opm.proxy.AccessApplicationProxy;
import com.huigou.util.ApplicationContextWrapper;
import com.huigou.util.StringUtil;

/**
 * 业务功能分组管理
 * 
 * @ClassName: BizFunctionApplicationImpl
 * @author
 * @date 2018-03-28 11:16
 * @version V1.0
 */
@Service("bizFunctionApplication")
public class BizFunctionApplicationImpl extends BaseApplication implements BizFunctionApplication {
    @Autowired
    private FunctionsRepository functionsRepository;

    @Autowired
    private FunctionsGroupRepository functionsGroupRepository;

    @Autowired
    private FunctionsDetailsRepository functionsDetailsRepository;

    @Autowired
    private AccessApplicationProxy accessApplication;

    @Override
    public Map<String, String> getGroupColorMap() {
        Map<String, String> map = new LinkedHashMap<String, String>();
        map.put("alert-success", "绿色");
        map.put("alert-info", "蓝色");
        map.put("alert-warning", "黄色");
        map.put("alert-danger", "红色");
        return map;
    }

    @Override
    public Map<String, String> getFuncColorMap() {
        Map<String, String> map = new LinkedHashMap<String, String>();
        map.put("btn-danger", "红色");
        map.put("btn-primary", "深蓝色");
        map.put("btn-warning", "黄色");
        map.put("btn-info", "浅蓝色");
        map.put("btn-success", "绿色");
        return map;
    }

    @Override
    @Transactional
    public String saveFunctions(Functions functions) {
        Assert.notNull(functions, CommonDomainConstants.OBJECT_NOT_NULL);
        functions = (Functions) this.commonDomainService.loadAndFillinProperties(functions);
        functions = (Functions) this.commonDomainService.saveBaseInfoEntity(functions, functionsRepository);
        return functions.getId();
    }

    @Override
    public Functions loadFunctions(String id) {
        Assert.hasText(id, CommonDomainConstants.ID_NOT_BLANK);
        return functionsRepository.findOne(id);
    }

    @Override
    @Transactional
    public void deleteFunctions(String id) {
        Assert.hasText(id, CommonDomainConstants.ID_NOT_BLANK);
        functionsRepository.delete(id);
        functionsGroupRepository.deleteByBpmFunctionsId(id);
        functionsDetailsRepository.deleteByBpmFunctionsId(id);
    }

    @Override
    public Map<String, Object> slicedQueryFunctions(FunctionsQueryRequest queryRequest) {
        QueryDescriptor queryDescriptor = this.sqlExecutorDao.getQuery(QUERY_XML_FILE_PATH, "functions");
        return this.sqlExecutorDao.executeSlicedQuery(queryDescriptor, queryRequest);
    }

    @Override
    @Transactional
    public String saveFunctionsGroup(FunctionsGroup functionsGroup) {
        Assert.notNull(functionsGroup, CommonDomainConstants.OBJECT_NOT_NULL);
        if (functionsGroup.isNew()) {
            functionsGroup.setSequence(this.commonDomainService.getNextSequence(FunctionsGroup.class, "bpmFunctionsId", functionsGroup.getBpmFunctionsId()));
        }
        functionsGroup = (FunctionsGroup) this.commonDomainService.loadAndFillinProperties(functionsGroup);
        functionsGroup = functionsGroupRepository.save(functionsGroup);
        return functionsGroup.getId();
    }

    @Override
    public FunctionsGroup loadFunctionsGroup(String id) {
        Assert.hasText(id, CommonDomainConstants.ID_NOT_BLANK);
        return functionsGroupRepository.findOne(id);
    }

    @Override
    @Transactional
    public void deleteFunctionsGroup(List<String> ids) {
        List<FunctionsGroup> objs = functionsGroupRepository.findAll(ids);
        for (FunctionsGroup obj : objs) {
            Integer count = functionsDetailsRepository.countByFunctionsGroupId(obj.getId());
            Assert.isTrue(count == 0, String.format("%s存在功能定义不能删除!", obj.getNameZh()));
        }
        functionsGroupRepository.delete(objs);
    }

    @Override
    public Map<String, Object> queryFunctionsGroup(FunctionsGroupQueryRequest queryRequest) {
        queryRequest.checkConstraints();
        QueryDescriptor queryDescriptor = this.sqlExecutorDao.getQuery(QUERY_XML_FILE_PATH, "functionsGroup");
        QueryModel model = this.sqlExecutorDao.getQueryModel(queryDescriptor, queryRequest);
        model.putDictionary("color", this.getGroupColorMap());
        return this.sqlExecutorDao.executeQuery(model);
    }

    @Override
    @Transactional
    public void updateFunctionsGroupSequence(Map<String, Integer> map) {
        Assert.notNull(map, MessageSourceContext.getMessage("object.not.null"));
        this.commonDomainService.updateSequence(FunctionsGroup.class, map);
    }

    @Override
    @Transactional
    public String saveFunctionsDetails(FunctionsDetails functionsDetails) {
        Assert.notNull(functionsDetails, CommonDomainConstants.OBJECT_NOT_NULL);
        if (functionsDetails.isNew()) {
            functionsDetails.setSequence(this.commonDomainService.getNextSequence(FunctionsDetails.class, "bpmFunctionsId",
                                                                                  functionsDetails.getBpmFunctionsId()));
        }
        functionsDetails = (FunctionsDetails) this.commonDomainService.loadAndFillinProperties(functionsDetails);
        functionsDetails = functionsDetailsRepository.save(functionsDetails);
        return functionsDetails.getId();
    }

    @Override
    public FunctionsDetails loadFunctionsDetails(String id) {
        Assert.hasText(id, CommonDomainConstants.ID_NOT_BLANK);
        return functionsDetailsRepository.findOne(id);
    }

    @Override
    @Transactional
    public void deleteFunctionsDetails(List<String> ids) {
        List<FunctionsDetails> objs = functionsDetailsRepository.findAll(ids);
        functionsDetailsRepository.delete(objs);
    }

    @Override
    public Map<String, Object> queryFunctionsDetails(FunctionsDetailsQueryRequest queryRequest) {
        QueryDescriptor queryDescriptor = this.sqlExecutorDao.getQuery(QUERY_XML_FILE_PATH, "functionsDetails");
        QueryModel model = this.sqlExecutorDao.getQueryModel(queryDescriptor, queryRequest);
        model.putDictionary("color", this.getFuncColorMap());
        return this.sqlExecutorDao.executeQuery(model);
    }

    @Override
    @Transactional
    public void updateFunctionsDetailsSequence(Map<String, Integer> map) {
        Assert.notNull(map, MessageSourceContext.getMessage("object.not.null"));
        this.commonDomainService.updateSequence(FunctionsDetails.class, map);
    }

    @Override
    @Transactional
    public void updateFunctionsDetailsGroup(List<String> ids, String groupId) {
        Assert.notNull(ids, MessageSourceContext.getMessage("object.not.null"));
        Assert.hasText(groupId, "groupId不能为空!");
        this.commonDomainService.move(FunctionsDetails.class, ids, "functions_group_id", groupId);
    }

    @Override
    @Transactional
    public void updateFunctionsDetailsColor(List<String> ids, String color) {
        Assert.notNull(ids, MessageSourceContext.getMessage("object.not.null"));
        Assert.hasText(color, "color不能为空!");
        this.commonDomainService.move(FunctionsDetails.class, ids, "color", color);
    }

    private List<FunctionsGroup> findGroupById(String id) {
        QueryDescriptor queryDescriptor = this.sqlExecutorDao.getQuery(QUERY_XML_FILE_PATH, "functionsGroup");
        String sql = queryDescriptor.getSqlByName("queryFunctionsGroupById");
        return this.sqlExecutorDao.queryToList(sql, FunctionsGroup.class, id);
    }

    private List<FunctionsDetails> findFunctionsDetailsById(String id) {
        QueryDescriptor queryDescriptor = this.sqlExecutorDao.getQuery(QUERY_XML_FILE_PATH, "functionsDetails");
        String sql = queryDescriptor.getSqlByName("queryFunctionsDetailsById");
        return this.sqlExecutorDao.queryToList(sql, FunctionsDetails.class, id);
    }

    @Override
    public List<FunctionsGroupDto> queryFunctionsGroup(String code) {
        Assert.hasText(code, "编码不能为空!");
        List<Functions> functions = functionsRepository.findByCode(code);
        Assert.notEmpty(functions, String.format("未找到[%s]对应的功能分组定义!", code));
        return this.queryFunctionsGroup(functions.get(0));
    }

    /**
     * 获取分组及权限
     * 
     * @param functions
     * @return
     */
    private List<FunctionsGroupDto> queryFunctionsGroup(Functions functions) {
        String beanName = functions.getCheckBeanName();
        List<FunctionsGroup> groups = this.findGroupById(functions.getId());
        List<FunctionsDetails> details = this.findFunctionsDetailsById(functions.getId());
        Locale locale = MessageSourceContext.getLocale();
        boolean isChina = locale.getLanguage().equalsIgnoreCase("zh");
        List<FunctionsGroupDto> groupDtos = new ArrayList<>(groups.size());
        for (FunctionsGroup group : groups) {
            FunctionsGroupDto dto = new FunctionsGroupDto();
            dto.setId(group.getId());
            dto.setColor(group.getColor());
            if (isChina) {
                dto.setName(group.getNameZh());
            } else {
                if (StringUtil.isNotBlank(group.getNameEn())) {
                    dto.setName(group.getNameEn());
                } else {
                    dto.setName(group.getNameZh());
                }
            }
            groupDtos.add(dto);
        }
        List<FunctionsDetailsDto> detailDtos = new ArrayList<>(details.size());
        for (FunctionsDetails detail : details) {
            FunctionsDetailsDto dto = new FunctionsDetailsDto();
            dto.setCode(detail.getCode());
            dto.setGroupId(detail.getFunctionsGroupId());
            dto.setColor(detail.getColor());
            dto.setIcon(detail.getIcon());
            dto.setSequence(detail.getSequence());
            if (isChina) {
                dto.setName(detail.getNameZh());
            } else {
                if (StringUtil.isNotBlank(detail.getNameEn())) {
                    dto.setName(detail.getNameEn());
                } else {
                    dto.setName(MessageSourceContext.getMessageAsDefault(String.format("function.%s.name", detail.getCode()), detail.getNameZh()));
                }
            }
            if (StringUtil.isBlank(detail.getUrl())) {
                dto.setUrl(detail.getFunUrl());
            } else {
                dto.setUrl(detail.getUrl());
            }
            detailDtos.add(dto);
        }
        FunctionGroupPermissionsApplication application = null;
        if (StringUtil.isNotBlank(beanName)) {
            application = ApplicationContextWrapper.getBean(beanName, FunctionGroupPermissionsApplication.class);
        }
        // 校验权限
        if (application == null) {
            detailDtos = this.checkFunctions(detailDtos);
        } else {
            detailDtos = application.checkFunctions(detailDtos);
        }
        // 组合功能到分组中
        for (FunctionsDetailsDto dto : detailDtos) {
            for (FunctionsGroupDto group : groupDtos) {
                if (StringUtil.isNotBlank(dto.getGroupId())) {
                    if (dto.getGroupId().equals(group.getId())) {
                        group.addFuns(dto);
                    }
                }
            }
        }
        return groupDtos;
    }

    /**
     * 默认系统权限校验
     * 
     * @param nodeFunctions
     * @return
     */
    private List<FunctionsDetailsDto> checkFunctions(List<FunctionsDetailsDto> datas) {
        Operator operator = ThreadLocalUtil.getOperator();
        List<FunctionsDetailsDto> objs = new ArrayList<>();
        if (datas == null || datas.size() == 0) {
            return objs;
        }
        for (FunctionsDetailsDto dto : datas) {
            dto.setHasPermission(accessApplication.checkPersonFunPermissions(operator.getUserId(), dto.getCode()));
            objs.add(dto);
        }
        return objs;
    }

}
