package com.huigou.bpmc.function.controller;

import java.util.List;
import java.util.Map;

import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.PathVariable;

import com.huigou.bpmc.function.application.BizFunctionApplication;
import com.huigou.bpmc.function.domain.model.Functions;
import com.huigou.bpmc.function.domain.model.FunctionsDetails;
import com.huigou.bpmc.function.domain.model.FunctionsGroup;
import com.huigou.bpmc.function.domain.query.FunctionsDetailsQueryRequest;
import com.huigou.bpmc.function.domain.query.FunctionsGroupQueryRequest;
import com.huigou.bpmc.function.domain.query.FunctionsQueryRequest;
import com.huigou.uasp.annotation.ControllerMapping;
import com.huigou.uasp.annotation.ControllerMethodMapping;
import com.huigou.uasp.client.CommonController;
import com.huigou.util.SDO;

/**
 * 业务功能分组管理
 * 
 * @ClassName: BizFunctionController
 * @author
 * @date 2018-03-28 11:16
 * @version V1.0
 */
@Controller
@ControllerMapping("bizFunction")
public class BizFunctionController extends CommonController {

    @Autowired
    private BizFunctionApplication bizFunctionApplication;

    protected String getPagePath() {
        return "/biz/bpmc/functions/";
    }

    @RequiresPermissions("BPMCBizFunction:query")
    public String forwardListFunctions() {
        return forward("functionsList");
    }

    @RequiresPermissions("BPMCBizFunction:query")
    public String slicedQueryFunctions() {
        SDO sdo = this.getSDO();
        FunctionsQueryRequest queryRequest = sdo.toQueryRequest(FunctionsQueryRequest.class);
        Map<String, Object> data = bizFunctionApplication.slicedQueryFunctions(queryRequest);
        return toResult(data);
    }

    @RequiresPermissions("BPMCBizFunction:create")
    public String showInsertFunctions() {
        return forward("functionsAdd");
    }

    @RequiresPermissions("BPMCBizFunction:create")
    public String insertFunctions() {
        SDO sdo = this.getSDO();
        Functions functions = sdo.toObject(Functions.class);
        String id = bizFunctionApplication.saveFunctions(functions);
        return success(id);
    }

    @RequiresPermissions("BPMCBizFunction:update")
    public String showLoadFunctions() {
        SDO sdo = this.getSDO();
        String id = sdo.getId();
        Functions functions = bizFunctionApplication.loadFunctions(id);
        return forward("functionsDetail", functions);
    }

    @RequiresPermissions("BPMCBizFunction:update")
    public String updateFunctions() {
        SDO sdo = this.getSDO();
        Functions functions = sdo.toObject(Functions.class);
        bizFunctionApplication.saveFunctions(functions);
        return success();
    }

    @RequiresPermissions("BPMCBizFunction:delete")
    public String deleteFunctions() {
        SDO sdo = this.getSDO();
        String id = sdo.getId();
        bizFunctionApplication.deleteFunctions(id);
        return success();
    }

    @RequiresPermissions("BPMCBizFunction:query")
    public String forwardListFunctionsGroup() {
        return forward("functionsGroupList");
    }

    @RequiresPermissions("BPMCBizFunction:query")
    public String queryFunctionsGroup() {
        SDO sdo = this.getSDO();
        FunctionsGroupQueryRequest queryRequest = sdo.toQueryRequest(FunctionsGroupQueryRequest.class);
        Map<String, Object> data = bizFunctionApplication.queryFunctionsGroup(queryRequest);
        return toResult(data);
    }

    @RequiresPermissions("BPMCBizFunction:create")
    public String showInsertFunctionsGroup() {
        this.putAttribute("groupColorList", bizFunctionApplication.getGroupColorMap());
        return forward("functionsGroupDetail");
    }

    @RequiresPermissions("BPMCBizFunction:create")
    public String insertFunctionsGroup() {
        SDO sdo = this.getSDO();
        FunctionsGroup functionsGroup = sdo.toObject(FunctionsGroup.class);
        String id = bizFunctionApplication.saveFunctionsGroup(functionsGroup);
        return success(id);
    }

    @RequiresPermissions("BPMCBizFunction:update")
    public String showLoadFunctionsGroup() {
        SDO sdo = this.getSDO();
        String id = sdo.getId();
        this.putAttribute("groupColorList", bizFunctionApplication.getGroupColorMap());
        FunctionsGroup functionsGroup = bizFunctionApplication.loadFunctionsGroup(id);
        return forward("functionsGroupDetail", functionsGroup);
    }

    public String showChooseFunctionsGroupColor() {
        this.putAttribute("groupColorList", bizFunctionApplication.getGroupColorMap());
        return forward("functionsGroupDetail");
    }

    @RequiresPermissions("BPMCBizFunction:update")
    public String updateFunctionsGroup() {
        SDO sdo = this.getSDO();
        FunctionsGroup functionsGroup = sdo.toObject(FunctionsGroup.class);
        bizFunctionApplication.saveFunctionsGroup(functionsGroup);
        return success();
    }

    @RequiresPermissions("BPMCBizFunction:delete")
    public String deleteFunctionsGroup() {
        SDO sdo = this.getSDO();
        List<String> ids = sdo.getIds();
        bizFunctionApplication.deleteFunctionsGroup(ids);
        return success();
    }

    @RequiresPermissions("BPMCBizFunction:update")
    public String updateFunctionsGroupSequence() {
        SDO sdo = this.getSDO();
        Map<String, Integer> map = sdo.getStringMap("data");
        bizFunctionApplication.updateFunctionsGroupSequence(map);
        return success();
    }

    @RequiresPermissions("BPMCBizFunction:query")
    public String forwardListFunctionsDetails() {
        return forward("functionsDetailsList");
    }

    @RequiresPermissions("BPMCBizFunction:query")
    public String queryFunctionsDetails() {
        SDO sdo = this.getSDO();
        FunctionsDetailsQueryRequest queryRequest = sdo.toQueryRequest(FunctionsDetailsQueryRequest.class);
        Map<String, Object> data = bizFunctionApplication.queryFunctionsDetails(queryRequest);
        return toResult(data);
    }

    @RequiresPermissions("BPMCBizFunction:create")
    public String showInsertFunctionsDetails() {
        this.putAttribute("functionColorList", bizFunctionApplication.getFuncColorMap());
        return forward("functionsFunDetail");
    }

    @RequiresPermissions("BPMCBizFunction:create")
    public String insertFunctionsDetails() {
        SDO sdo = this.getSDO();
        FunctionsDetails functionsDetails = sdo.toObject(FunctionsDetails.class);
        String id = bizFunctionApplication.saveFunctionsDetails(functionsDetails);
        return success(id);
    }

    @RequiresPermissions("BPMCBizFunction:update")
    public String showLoadFunctionsDetails() {
        SDO sdo = this.getSDO();
        String id = sdo.getId();
        FunctionsDetails functionsDetails = bizFunctionApplication.loadFunctionsDetails(id);
        this.putAttribute("functionColorList", bizFunctionApplication.getFuncColorMap());
        return forward("functionsFunDetail", functionsDetails);
    }

    @RequiresPermissions("BPMCBizFunction:update")
    public String updateFunctionsDetails() {
        SDO sdo = this.getSDO();
        FunctionsDetails functionsDetails = sdo.toObject(FunctionsDetails.class);
        bizFunctionApplication.saveFunctionsDetails(functionsDetails);
        return success();
    }

    public String showChooseFunctionsDetailColor() {
        this.putAttribute("functionColorList", bizFunctionApplication.getFuncColorMap());
        return forward("functionsFunDetailColor");
    }

    @RequiresPermissions("BPMCBizFunction:delete")
    public String deleteFunctionsDetails() {
        SDO sdo = this.getSDO();
        List<String> ids = sdo.getIds();
        bizFunctionApplication.deleteFunctionsDetails(ids);
        return success();
    }

    @RequiresPermissions("BPMCBizFunction:update")
    public String updateFunctionsDetailsSequence() {
        SDO sdo = this.getSDO();
        Map<String, Integer> map = sdo.getStringMap("data");
        bizFunctionApplication.updateFunctionsDetailsSequence(map);
        return success();
    }

    @RequiresPermissions("BPMCBizFunction:update")
    public String updateFunctionsDetailsGroup() {
        SDO sdo = this.getSDO();
        String groupId = sdo.getString("groupId");
        List<String> ids = sdo.getIds();
        bizFunctionApplication.updateFunctionsDetailsGroup(ids, groupId);
        return success();
    }

    @RequiresPermissions("BPMCBizFunction:update")
    public String updateFunctionsDetailsColor() {
        SDO sdo = this.getSDO();
        String color = sdo.getString("color");
        List<String> ids = sdo.getIds();
        bizFunctionApplication.updateFunctionsDetailsColor(ids, color);
        return success();
    }

    public String showFunctionsGroup() {
        SDO sdo = this.getSDO();
        String code = sdo.getString("code");
        this.putAttribute("functionGroupList", bizFunctionApplication.queryFunctionsGroup(code));
        return forward("functionsGroup");
    }

    /**
     * RESTful 风格调用
     * 
     * @param code
     * @return
     */
    @ControllerMethodMapping(value = "/{code}/show", combine = true)
    public String show(@PathVariable("code") String code) {
        this.putAttribute("functionGroupList", bizFunctionApplication.queryFunctionsGroup(code));
        return forward("functionsGroup");
    }

}
