package com.huigou.bpmc.process.controller;

import java.util.List;
import java.util.Map;

import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;

import com.huigou.bpmc.process.application.BPMCBusinessProcessApplication;
import com.huigou.bpmc.process.domain.model.BusinessProcess;
import com.huigou.bpmc.process.domain.query.BusinessProcessQueryRequest;
import com.huigou.data.domain.model.CommonDomainConstants;
import com.huigou.uasp.annotation.ControllerMapping;
import com.huigou.uasp.client.CommonController;
import com.huigou.util.SDO;

@Controller
@ControllerMapping("bpmcBusinessProcess")
public class BPMCBusinessProcessController extends CommonController {
    private static final String BUSINESS_PROCESS_LIST_PAGE = "businessProcessList";

    private static final String BUSINESS_PROCESS_EDIT_PAGE = "businessProcessEdit";

    private static final String BUSINESS_PROCESS_DETAIL_PAGE = "businessProcessDetail";

    @Autowired
    private BPMCBusinessProcessApplication bpmBusinessProcessApplication;

    protected String getPagePath() {
        return "/biz/bpmc/process/";
    }

    @RequiresPermissions("BPMCBusinessProcess:query")
    public String forwardBusinessProcess() {
        return forward(BUSINESS_PROCESS_LIST_PAGE);
    }

    @RequiresPermissions("BPMCBusinessProcess:create")
    public String showInsertBusinessProcess() {
        SDO sdo = this.getSDO();
        String parentId = sdo.getString("parentId");
        Integer sequence = bpmBusinessProcessApplication.getBusinessProcessNextSequence(parentId);
        if (!parentId.equals(CommonDomainConstants.DEFAULT_ROOT_PARENT_ID)) {
            BusinessProcess businessProcess = bpmBusinessProcessApplication.loadBusinessProcess(parentId);
            sdo.putProperty("parentName", businessProcess.getName());
            sdo.putProperty("parentCode", businessProcess.getCode());
        }
        sdo.putProperty("sequence", sequence);
        return forward(BUSINESS_PROCESS_DETAIL_PAGE, sdo);
    }

    @RequiresPermissions("BPMCBusinessProcess:update")
    public String showUpdateBusinessProcess() {
        SDO sdo = this.getSDO();
        String id = sdo.getId();
        BusinessProcess businessProcess = bpmBusinessProcessApplication.loadBusinessProcess(id);
        return forward(BUSINESS_PROCESS_DETAIL_PAGE, businessProcess);
    }

    @RequiresPermissions("BPMCBusinessProcess:update")
    public String showEditBusinessProcessAttribute() {
        SDO sdo = this.getSDO();
        String id = sdo.getId();
        BusinessProcess businessProcess = bpmBusinessProcessApplication.loadBusinessProcess(id);
        return forward(BUSINESS_PROCESS_EDIT_PAGE, businessProcess);
    }

    @RequiresPermissions("BPMCBusinessProcess:create")
    public String insertBusinessProcess() {
        SDO sdo = this.getSDO();
        BusinessProcess businessProcess = sdo.toObject(BusinessProcess.class);
        String id = bpmBusinessProcessApplication.insertBusinessProcess(businessProcess);
        return success(id);
    }

    @RequiresPermissions("BPMCBusinessProcess:update")
    public String saveBusinessProcessAndAttribute() {
        SDO sdo = this.getSDO();
        BusinessProcess businessProcess = sdo.toObject(BusinessProcess.class);
        bpmBusinessProcessApplication.saveBusinessProcessAndAttribute(businessProcess);
        return success();
    }

    @RequiresPermissions("BPMCBusinessProcess:update")
    public String updateBusinessProcess() {
        SDO sdo = this.getSDO();
        BusinessProcess businessProcess = sdo.toObject(BusinessProcess.class);
        bpmBusinessProcessApplication.updateBusinessProcess(businessProcess);
        return success();
    }

    @RequiresPermissions("BPMCBusinessProcess:update")
    public String updateBusinessProcessesSequence() {
        SDO sdo = this.getSDO();
        String parentId = sdo.getString(PARENT_ID_KEY_NAME);
        Map<String, Integer> params = sdo.getStringMap("data");
        this.bpmBusinessProcessApplication.updateBusinessProcessesSequence(parentId, params);
        return success();
    }

    @RequiresPermissions("BPMCBusinessProcess:delete")
    public String deleteBusinessProcesses() {
        SDO sdo = this.getSDO();
        List<String> ids = sdo.getStringList(IDS_KEY_NAME);
        bpmBusinessProcessApplication.deleteBusinessProcesses(ids);
        return toResult("");
    }

    public String slicedQueryBusinessProcesses() {
        SDO sdo = this.getSDO();
        BusinessProcessQueryRequest queryRequest = sdo.toQueryRequest(BusinessProcessQueryRequest.class);
        Map<String, Object> data = bpmBusinessProcessApplication.slicedQueryBusinessProcesses(queryRequest);
        return this.toResult(data);
    }

    public String queryBusinessProcesses() {
        SDO sdo = this.getSDO();
        String parentId = sdo.getString(PARENT_ID_KEY_NAME);
        List<Map<String, Object>> data = bpmBusinessProcessApplication.queryBusinessProcesses(parentId);
        return this.toResult(data);
    }

    public String queryBusinessProcessesOnMove() {
        SDO sdo = this.getSDO();
        String parentId = sdo.getString(PARENT_ID_KEY_NAME);
        String excludeIds = sdo.getString("excludeIds");
        List<Map<String, Object>> data = bpmBusinessProcessApplication.queryBusinessProcessesOnMove(parentId, excludeIds);
        return this.toResult(data);
    }

    @RequiresPermissions("BPMCBusinessProcess:update")
    public String moveBusinessProcesses() {
        SDO params = this.getSDO();
        String parentId = params.getString("parentId");
        List<String> ids = params.getStringList("ids");
        this.bpmBusinessProcessApplication.moveBusinessProcesses(parentId, ids);
        return this.success();
    }
}
