package com.huigou.uasp.tool;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.collections.MapUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import com.huigou.cache.SystemCache;
import com.huigou.context.MessageSourceContext;
import com.huigou.data.domain.query.EmptyQueryRequest;
import com.huigou.data.query.model.QueryDescriptor;
import com.huigou.data.query.model.QueryModel;
import com.huigou.uasp.bmp.fn.AbstractDaoFunction;
import com.huigou.uasp.bmp.opm.proxy.OrgApplicationProxy;
import com.huigou.util.ClassHelper;
import com.huigou.util.StringUtil;
import com.itextpdf.text.Image;

@Service("pdfService")
public class PDFService extends AbstractDaoFunction {

    @Autowired
    protected OrgApplicationProxy orgApplication;

    public Map<String,Object> getOrgProperties(String orgId, String description, Boolean isQueryParents){
        Assert.isTrue(StringUtil.isNotBlank(orgId), MessageSourceContext.getMessage("fieldName.not.null", "orgId"));

        Map<String,Object> result = new HashMap<>();

        QueryDescriptor queryDescriptor = this.sqlExecutorDao.getQuery("config/tech/common/techCommonFunction.xml", "orgPropertiesCommonFunction");
        QueryModel queryModel = this.sqlExecutorDao.getQueryModel(queryDescriptor, new EmptyQueryRequest());

        StringBuffer orgQuerySql=new StringBuffer();
        if(!isQueryParents) {//如果不查询父级org
            orgQuerySql.append("t.orgId = '").append(orgId).append("'");
        }else{ //查询父级
            String sql  = queryDescriptor.getSqlByName("queryInCludeParents");
            orgQuerySql = orgQuerySql.append("'").append(orgId).append("'");
            sql = sql.replace(":orgId",orgQuerySql.toString());
            orgQuerySql = new StringBuffer("EXISTS (");
            orgQuerySql.append(sql).append(")");
        }
        queryModel.setSql(queryModel.getSql().replace(":orgQuerySql",orgQuerySql.toString()));
        queryModel.putParam("propertyeDesciption",description);

        //此处采取趋近选取策略，这两个排序条件和排序方式均不可更改
        String sortFiedName= "LENGTH(full_id)",sortOrder = "asc";
        queryModel.setSortFieldName(sortFiedName);
        queryModel.setSortOrder(sortOrder);

        result = this.sqlExecutorDao.executeQuery(queryModel);

        return result;
    }

    public Map<String,Object> getOrgProperties(String orgId){
        return getOrgProperties(orgId,null,false);
    }

    public Map<String,Object> getOrgPropertiesIncludeParents(String orgId){
        return getOrgProperties(orgId,null,true);
    }

    public Map<String,Object> getOrgProperty(String orgId,String description){
        return getOrgProperties(orgId,description,false);
    }

    public Map<String,Object> getOrgPropertyIncludeParents(String orgId,String description){
        return getOrgProperties(orgId,description,true);
    }

    public <T> T getOrgProperty(String orgId, String description, Class<T> clazz, Boolean... isQueryParents){
        Boolean flag = isQueryParents!=null && isQueryParents.length>0 && isQueryParents[0];
        Map<String,Object> result = convertPropRow2Map(getOrgProperties(orgId,description,flag));
        return ClassHelper.convert(result.get(description),clazz);
    }

    public <T> T getOrgPropertyIncludeParents(String orgId, String description, Class<T> clazz){
        Map<String,Object> result = convertPropRow2Map(getOrgProperties(orgId,description,true));
        return ClassHelper.convert(result.get(description),clazz);
    }

    public Map<String,Object> convertPropRow2Map(Map<String,Object> rows){
        return convertPropRow2Map(rows,false);
    }

    public Map<String,Object> convertPropRow2Map(Map<String,Object> rows,Boolean isFull){
        Map<String,Object> result = new HashMap<>();
        if(rows!=null && MapUtils.getInteger(rows,"Total",0)>0){
            List<Map<String,Object>> propList = (List<Map<String,Object>>)rows.get("Rows");
            for(Map<String,Object> prop : propList){
                String key = MapUtils.getString(prop, "propertyDescription");
                if(isFull){
                    result.put(key,prop);
                }else{
                    result.put(key, MapUtils.getObject(prop, "propertyValue"));
                }
            }
        }
        return result;
    }

    //获取系统参数中配置的值
    public Map<String,Object> getPDFParameter(){
        String[] pdfParameterCode = new String[]{"fillOpacity","imgLeft","imgTop","rotation","rotationDegrees","scaleAbsoluteWidth","scaleAbsoluteHeight","scalePercent","underOrOver","alignment"};
        String[] pdfParameterDefaultValue = new String[]{"1f","0f","0f","0","0",null,null,"100","over", String.valueOf(Image.ALIGN_TOP)};
        Map<String,Object> result = new HashMap<>();
        for(int i= 0;i< pdfParameterCode.length ; i++){
            String code = pdfParameterCode[i];
            String value = SystemCache.getParameter("watermark."+code,String.class);
            if(StringUtil.isBlank(value)) value = pdfParameterDefaultValue[i]; //设置默认值
            result.put(code,value);
        }
        return result;
    }
}
