package com.huigou.uasp.tool;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Map;

import com.huigou.context.ThreadLocalUtil;
import org.apache.commons.collections.MapUtils;
import org.springframework.beans.factory.annotation.Autowired;

import com.huigou.cache.SystemCache;
import com.huigou.uasp.bmp.doc.attachment.controller.AttachmentController;
import com.huigou.uasp.client.OutputFileInterceptor;
import com.huigou.util.StringUtil;
import com.itextpdf.text.Image;
import com.itextpdf.text.pdf.PdfContentByte;
import com.itextpdf.text.pdf.PdfGState;
import com.itextpdf.text.pdf.PdfReader;
import com.itextpdf.text.pdf.PdfStamper;

/**
 * PDF添加水印。
 */
public class PdfWatermarkOutputFileInterceptor implements OutputFileInterceptor {

    private PDFService pdfService;

    @Autowired
    public void setPdfService(PDFService pdfService) {
        this.pdfService = pdfService;
    }

    @Override
    public InputStream intercept(InputStream is, String type, Object target) throws IOException {
        if(target instanceof AttachmentController) {
            // 附件预览不需要添加水印
            return is;
        }
        //用款打印增加财务联和资产联的水印
        String printMoney = (String)ThreadLocalUtil.getVariable("printMoney");
        String markImagePath2="";
        String markImagePath3="";
        String isNeedWatermark = SystemCache.getParameter("isNeedWatermark", String.class); //读取系统是否需要水印
        if ("pdf".equalsIgnoreCase(type) && "yes".equalsIgnoreCase(isNeedWatermark)) {
//            String markImagePath = pdfService.getOrgPropertyIncludeParents(ThreadLocalUtil.getOperator().getPersonMemberId(), "markImagePath", String.class);
            
            String markImagePath = "images/ssrf_watermark.png";
            if (StringUtil.isNotBlank(markImagePath) && !"none".equalsIgnoreCase(markImagePath)) { //设置为"none"字符串会被认定为黑名单中
                Map<String, Object> pdfParameter = pdfService.getPDFParameter();
                if ("printMoneyZiChan".equals(printMoney)){//资产联水印
                    Map<String, Object> pdfParameter2 = pdfService.getPDFParameter();
                    //markImagePath2 = "images/zichanlian_watermark.png";
                    markImagePath2 = "images/zichan.png";
                    pdfParameter2.put("markImagePath2", SystemCache.getRealPath() + markImagePath2);
                    pdfParameter.put("markImagePath", SystemCache.getRealPath() + markImagePath);
                    return addPdfImgMarkMoneyZiChan(is, pdfParameter, pdfParameter2);
                }else if ("printMoneyCaiWu".equals(printMoney)){
                    Map<String, Object> pdfParameter3 = pdfService.getPDFParameter();
                    //markImagePath3 = "images/caiwulian_watermark.png";
                    markImagePath3 = "images/caiwu.png";
                    pdfParameter3.put("markImagePath3", SystemCache.getRealPath() + markImagePath3);
                    pdfParameter.put("markImagePath", SystemCache.getRealPath() + markImagePath);
                    return addPdfImgMarkMoneyCaiWu(is, pdfParameter, pdfParameter3);
                }else {
                    pdfParameter.put("markImagePath", SystemCache.getRealPath() + markImagePath);
                    return addPdfImgMark(is, pdfParameter);
                }
            }
        }
        ThreadLocalUtil.removeVariable("printMoney");
        return is;
    }

    /**
     * <br>
     * <p>
     * Description: 给pdf文件添加水印 <br>
     * <p><br/>
     * <p>
     *
     * @param is    要加水印的原pdf文件路径
     * @param param 水印参数参数
     */
    private InputStream addPdfImgMark(InputStream is, Map<String, Object> param) throws IOException {
        PdfReader reader = null;
        PdfStamper stamp = null;
        ByteArrayOutputStream bos = new ByteArrayOutputStream(1024);
        try {
            reader = new PdfReader(is, "PDF".getBytes());
            stamp = new PdfStamper(reader, bos);

            PdfContentByte mask; //生成遮罩

            PdfGState gs1 = new PdfGState();
            gs1.setFillOpacity(MapUtils.getFloat(param, "fillOpacity"));// 透明度设置

            Image img = Image.getInstance(MapUtils.getString(param, "markImagePath"));// 插入图片水印

            int rotation = MapUtils.getIntValue(param, "rotation", 0), rotationDegrees = MapUtils.getIntValue(param, "rotationDegrees", 0), scaleAbsoluteWidth = MapUtils.getIntValue(param, "scaleAbsoluteWidth", 0), scaleAbsoluteHeight = MapUtils.getIntValue(param, "scaleAbsoluteHeight", 0), scalePercent = MapUtils.getIntValue(param, "scalePercent", 0);
            if (rotation != 0) {
                img.setRotation(rotation);// 旋转 弧度
            }
            if (rotationDegrees != 0) {
                img.setRotationDegrees(rotationDegrees);// 旋转 角度
            }
            if (scaleAbsoluteWidth != 0 && scaleAbsoluteHeight != 0) {
                img.scaleAbsolute(scaleAbsoluteWidth, scaleAbsoluteHeight);//自定义大小
            }
            if (scalePercent != 0) {
                img.scalePercent(scalePercent);//依照比例缩放
            }

            int pageSize = reader.getNumberOfPages();// 原pdf文件的总页数
            String underOrOver = MapUtils.getString(param, "underOrOver");//水印在文本下或文本上

            img.setAbsolutePosition(MapUtils.getFloat(param, "imgLeft"), MapUtils.getFloat(param, "imgTop")); // 坐标
            //img.setAlignment(MapUtils.getIntValue(param,"alignment"));
            Float imgLeft = MapUtils.getFloat(param, "imgLeft"), imgTop = MapUtils.getFloat(param, "imgTop");
            for (int i = 1; i <= pageSize; i++) {
                if ("under".equalsIgnoreCase(underOrOver)) {
                    mask = stamp.getUnderContent(i);// 水印在之前文本下
                } else {
                    mask = stamp.getOverContent(i);//水印在之前文本上
                }
                mask.setGState(gs1);// 图片水印 透明度
                mask.addImage(img);// 图片水印
            }
            stamp.close();// 关闭
            return new ByteArrayInputStream(bos.toByteArray());
        } catch (Exception ex) {
            throw new IOException("给pdf文件添加水印失败", ex);
        } finally {
            if (reader != null) {
                reader.close();
            }
            if (stamp != null) {
                try {
                    stamp.close();
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        }
    }

    /**
     * <br>
     * <p>
     * Description: 给pdf文件添加水印 <br>
     * <p><br/>
     * <p>
     *
     * @param is    要加水印的原pdf文件路径
     * @param param2 资产联水印参数参数
     */
    private InputStream addPdfImgMarkMoneyZiChan(InputStream is,Map<String, Object> param, Map<String, Object> param2) throws IOException {
        PdfReader reader = null;
        PdfStamper stamp = null;
        ByteArrayOutputStream bos = new ByteArrayOutputStream(1024);
        try {
            reader = new PdfReader(is, "PDF".getBytes());
            stamp = new PdfStamper(reader, bos);

            PdfContentByte mask; //生成遮罩
            PdfContentByte mask2; //生成遮罩

            PdfGState gs1 = new PdfGState();
            gs1.setFillOpacity(MapUtils.getFloat(param2, "fillOpacity"));// 透明度设置

            PdfGState gs12 = new PdfGState();
            gs12.setFillOpacity(0.6f);// 透明度设置

            Image img2 = Image.getInstance(MapUtils.getString(param2, "markImagePath2"));// 插入图片水印
            Image img = Image.getInstance(MapUtils.getString(param, "markImagePath"));// 插入图片水印

            int rotation = MapUtils.getIntValue(param, "rotation", 0), rotationDegrees = MapUtils.getIntValue(param, "rotationDegrees", 0), scaleAbsoluteWidth = MapUtils.getIntValue(param, "scaleAbsoluteWidth", 0), scaleAbsoluteHeight = MapUtils.getIntValue(param, "scaleAbsoluteHeight", 0), scalePercent = MapUtils.getIntValue(param, "scalePercent", 0);
            int rotation2 = MapUtils.getIntValue(param2, "rotation2", 0), rotationDegrees2 = 0, scaleAbsoluteWidth2 = MapUtils.getIntValue(param2, "scaleAbsoluteWidth", 0), scaleAbsoluteHeight2 = MapUtils.getIntValue(param2, "scaleAbsoluteHeight", 0), scalePercent2 = MapUtils.getIntValue(param2, "scalePercent", 0);
            if (rotation2 != 0) {
                img2.setRotation(rotation2);// 旋转 弧度
                img.setRotation(rotation);// 旋转 弧度
            }
            //if (rotationDegrees2 != 0) {
                img2.setRotationDegrees(rotationDegrees2);// 旋转 角度
            //}
            if (rotationDegrees != 0) {
            img.setRotationDegrees(rotationDegrees);// 旋转 角度
            }
            if (scaleAbsoluteWidth2 != 0 && scaleAbsoluteHeight2 != 0) {
                img2.scaleAbsolute(scaleAbsoluteWidth2, scaleAbsoluteHeight2);//自定义大小
                img.scaleAbsolute(scaleAbsoluteWidth, scaleAbsoluteHeight);//自定义大小
            }
            if (scalePercent2 != 0) {
                img2.scalePercent(scalePercent2);//依照比例缩放
                img.scalePercent(scalePercent);//依照比例缩放
            }

            int pageSize = reader.getNumberOfPages();// 原pdf文件的总页数
            String underOrOver = MapUtils.getString(param2, "underOrOver");//水印在文本下或文本上

            img.setAbsolutePosition(MapUtils.getFloat(param, "imgLeft"), MapUtils.getFloat(param, "imgTop")); // 坐标
            img2.setAbsolutePosition(415, 765); // 坐标
            for (int i = 1; i <= pageSize; i++) {
                if ("under".equalsIgnoreCase(underOrOver)) {
                    mask2 = stamp.getUnderContent(i);// 水印在之前文本下
                    mask = stamp.getUnderContent(i);// 水印在之前文本下
                } else {
                    //mask2 = stamp.getOverContent(i);//水印在之前文本上
                    //mask = stamp.getOverContent(i);//水印在之前文本上
                    mask2 = stamp.getUnderContent(i);// 水印在之前文本下
                    mask = stamp.getUnderContent(i);// 水印在之前文本下
                }
                mask2.setGState(gs12);// 图片水印 透明度
                mask2.addImage(img2);// 图片水印
                mask.setGState(gs1);// 图片水印 透明度
                mask.addImage(img);// 图片水印
            }
            stamp.close();// 关闭
            return new ByteArrayInputStream(bos.toByteArray());
        } catch (Exception ex) {
            throw new IOException("给pdf文件添加水印失败", ex);
        } finally {
            if (reader != null) {
                reader.close();
            }
            if (stamp != null) {
                try {
                    stamp.close();
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        }
    }

    /**
     * <br>
     * <p>
     * Description: 给pdf文件添加水印 <br>
     * <p><br/>
     * <p>
     *
     * @param is    要加水印的原pdf文件路径
     * @param param3 财务联水印参数参数
     */
    private InputStream addPdfImgMarkMoneyCaiWu(InputStream is, Map<String, Object> param, Map<String, Object> param3) throws IOException {
        PdfReader reader = null;
        PdfStamper stamp = null;
        ByteArrayOutputStream bos = new ByteArrayOutputStream(1024);
        try {
            reader = new PdfReader(is, "PDF".getBytes());
            stamp = new PdfStamper(reader, bos);

            PdfContentByte mask3; //生成遮罩
            PdfContentByte mask; //生成遮罩

            PdfGState gs1 = new PdfGState();
            gs1.setFillOpacity(MapUtils.getFloat(param3, "fillOpacity"));// 透明度设置

            PdfGState gs13 = new PdfGState();
            gs13.setFillOpacity(0.6f);// 透明度设置

            Image img3 = Image.getInstance(MapUtils.getString(param3, "markImagePath3"));// 插入图片水印
            Image img = Image.getInstance(MapUtils.getString(param, "markImagePath"));// 插入图片水印

            int rotation = MapUtils.getIntValue(param, "rotation", 0), rotationDegrees = MapUtils.getIntValue(param, "rotationDegrees", 0), scaleAbsoluteWidth = MapUtils.getIntValue(param, "scaleAbsoluteWidth", 0), scaleAbsoluteHeight = MapUtils.getIntValue(param, "scaleAbsoluteHeight", 0), scalePercent = MapUtils.getIntValue(param, "scalePercent", 0);
            int rotation3 = MapUtils.getIntValue(param3, "rotation3", 0), rotationDegrees3 = 0, scaleAbsoluteWidth3 = MapUtils.getIntValue(param3, "scaleAbsoluteWidth", 0), scaleAbsoluteHeight3 = MapUtils.getIntValue(param3, "scaleAbsoluteHeight", 0), scalePercent3 = MapUtils.getIntValue(param3, "scalePercent", 0);
            if (rotation3 != 0) {
                img3.setRotation(rotation3);// 旋转 弧度
                img.setRotation(rotation);// 旋转 弧度
            }
            if (rotationDegrees != 0) {
                img.setRotationDegrees(rotationDegrees);// 旋转 角度
            }
            img3.setRotationDegrees(rotationDegrees3);// 旋转 角度
            if (scaleAbsoluteWidth3 != 0 && scaleAbsoluteHeight3 != 0) {
                img3.scaleAbsolute(scaleAbsoluteWidth3, scaleAbsoluteHeight3);//自定义大小
                img.scaleAbsolute(scaleAbsoluteWidth, scaleAbsoluteHeight);//自定义大小
            }
            if (scalePercent3 != 0) {
                img3.scalePercent(scalePercent3);//依照比例缩放
                img.scalePercent(scalePercent);//依照比例缩放
            }

            int pageSize = reader.getNumberOfPages();// 原pdf文件的总页数
            String underOrOver = MapUtils.getString(param3, "underOrOver");//水印在文本下或文本上

            img.setAbsolutePosition(MapUtils.getFloat(param, "imgLeft"), MapUtils.getFloat(param, "imgTop")); // 坐标
            img3.setAbsolutePosition(415, 765); // 坐标
            for (int i = 1; i <= pageSize; i++) {
                if ("under".equalsIgnoreCase(underOrOver)) {
                    mask3 = stamp.getUnderContent(i);// 水印在之前文本下
                    mask = stamp.getUnderContent(i);// 水印在之前文本下
                } else {
                    //mask3 = stamp.getOverContent(i);//水印在之前文本上
                    //mask = stamp.getOverContent(i);//水印在之前文本上
                    mask3 = stamp.getUnderContent(i);// 水印在之前文本下
                    mask = stamp.getUnderContent(i);// 水印在之前文本下
                }
                mask3.setGState(gs13);// 图片水印 透明度
                mask3.addImage(img3);// 图片水印
                mask.setGState(gs1);// 图片水印 透明度
                mask.addImage(img);// 图片水印
            }
            stamp.close();// 关闭
            return new ByteArrayInputStream(bos.toByteArray());
        } catch (Exception ex) {
            throw new IOException("给pdf文件添加水印失败", ex);
        } finally {
            if (reader != null) {
                reader.close();
            }
            if (stamp != null) {
                try {
                    stamp.close();
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        }
    }
}
